From 13417cf9c6eec555b51da49742e47939d2f5715b Mon Sep 17 00:00:00 2001
From: James Moger <james.moger@gitblit.com>
Date: Fri, 19 Oct 2012 22:47:33 -0400
Subject: [PATCH] Exclude submodules from zip downloads (issue 151)

---
 src/com/gitblit/wicket/pages/EditRepositoryPage.java |   86 ++++++++++++++++++++++++++++++++++++-------
 1 files changed, 72 insertions(+), 14 deletions(-)

diff --git a/src/com/gitblit/wicket/pages/EditRepositoryPage.java b/src/com/gitblit/wicket/pages/EditRepositoryPage.java
index 572f650..a2dad10 100644
--- a/src/com/gitblit/wicket/pages/EditRepositoryPage.java
+++ b/src/com/gitblit/wicket/pages/EditRepositoryPage.java
@@ -36,6 +36,8 @@
 import org.apache.wicket.markup.html.form.DropDownChoice;
 import org.apache.wicket.markup.html.form.Form;
 import org.apache.wicket.markup.html.form.IChoiceRenderer;
+import org.apache.wicket.markup.html.form.Radio;
+import org.apache.wicket.markup.html.form.RadioGroup;
 import org.apache.wicket.markup.html.form.TextField;
 import org.apache.wicket.markup.html.list.ListItem;
 import org.apache.wicket.markup.html.list.ListView;
@@ -47,6 +49,7 @@
 
 import com.gitblit.Constants;
 import com.gitblit.Constants.AccessRestrictionType;
+import com.gitblit.Constants.AuthorizationControl;
 import com.gitblit.Constants.FederationStrategy;
 import com.gitblit.GitBlit;
 import com.gitblit.GitBlitException;
@@ -75,6 +78,21 @@
 		RepositoryModel model = new RepositoryModel();
 		String restriction = GitBlit.getString(Keys.git.defaultAccessRestriction, null);
 		model.accessRestriction = AccessRestrictionType.fromName(restriction);
+		String authorization = GitBlit.getString(Keys.git.defaultAuthorizationControl, null);
+		model.authorizationControl = AuthorizationControl.fromName(authorization);
+		
+		GitBlitWebSession session = GitBlitWebSession.get();
+		UserModel user = session.getUser();
+		if (user != null && user.canCreate() && !user.canAdmin()) {
+			// personal create permissions, inject personal repository path
+			model.name = user.getPersonalPath() + "/";
+			model.projectPath = user.getPersonalPath();
+			model.owner = user.username;
+			// personal repositories are private by default
+			model.accessRestriction = AccessRestrictionType.VIEW;
+			model.authorizationControl = AuthorizationControl.NAMED;
+		}
+		
 		setupPage(model);
 	}
 
@@ -98,8 +116,15 @@
 		List<String> preReceiveScripts = new ArrayList<String>();
 		List<String> postReceiveScripts = new ArrayList<String>();
 
+		GitBlitWebSession session = GitBlitWebSession.get();
+		final UserModel user = session.getUser() == null ? UserModel.ANONYMOUS : session.getUser();
+
 		if (isCreate) {
-			super.setupPage(getString("gb.newRepository"), "");
+			if (user.canAdmin()) {
+				super.setupPage(getString("gb.newRepository"), "");
+			} else {
+				super.setupPage(getString("gb.newRepository"), user.getDisplayName());
+			}
 		} else {
 			super.setupPage(getString("gb.edit"), repositoryModel.name);
 			if (repositoryModel.accessRestriction.exceeds(AccessRestrictionType.NONE)) {
@@ -190,11 +215,14 @@
 			protected void onSubmit() {
 				try {
 					// confirm a repository name was entered
-					if (StringUtils.isEmpty(repositoryModel.name)) {
+					if (repositoryModel.name == null && StringUtils.isEmpty(repositoryModel.name)) {
 						error(getString("gb.pleaseSetRepositoryName"));
 						return;
 					}
-
+					
+					// ensure name is trimmed
+					repositoryModel.name = repositoryModel.name.trim();
+					
 					// automatically convert backslashes to forward slashes
 					repositoryModel.name = repositoryModel.name.replace('\\', '/');
 					// Automatically replace // with /
@@ -212,6 +240,9 @@
 					if (repositoryModel.name.contains("/../")) {
 						error(getString("gb.illegalRelativeSlash"));
 						return;
+					}					
+					if (repositoryModel.name.endsWith("/")) {
+						repositoryModel.name = repositoryModel.name.substring(0, repositoryModel.name.length() - 1);
 					}
 
 					// confirm valid characters in repository name
@@ -220,6 +251,22 @@
 						error(MessageFormat.format(getString("gb.illegalCharacterRepositoryName"),
 								c));
 						return;
+					}
+					
+					if (user.canCreate() && !user.canAdmin()) {
+						// ensure repository name begins with the user's path
+						if (!repositoryModel.name.startsWith(user.getPersonalPath())) {
+							error(MessageFormat.format(getString("gb.illegalPersonalRepositoryLocation"),
+									user.getPersonalPath()));
+							return;
+						}
+						
+						if (repositoryModel.name.equals(user.getPersonalPath())) {
+							// reset path prefix and show error
+							repositoryModel.name = user.getPersonalPath() + "/";
+							error(getString("gb.pleaseSetRepositoryName"));
+							return;
+						}
 					}
 
 					// confirm access restriction selection
@@ -331,10 +378,11 @@
 		form.add(new SimpleAttributeModifier("autocomplete", "off"));
 
 		// field names reflective match RepositoryModel fields
-		form.add(new TextField<String>("name").setEnabled(isCreate || isAdmin));
+		form.add(new TextField<String>("name").setEnabled(isCreate || isAdmin || repositoryModel.isUsersPersonalRepository(user.username)));
 		form.add(new TextField<String>("description"));
 		form.add(new DropDownChoice<String>("owner", GitBlit.self().getAllUsernames())
 				.setEnabled(GitBlitWebSession.get().canAdmin()));
+		form.add(new CheckBox("allowForks"));
 		form.add(new DropDownChoice<AccessRestrictionType>("accessRestriction", Arrays
 				.asList(AccessRestrictionType.values()), new AccessRestrictionRenderer()));
 		form.add(new CheckBox("isFrozen"));
@@ -358,7 +406,7 @@
 		form.add(new DropDownChoice<FederationStrategy>("federationStrategy", federationStrategies,
 				new FederationTypeRenderer()));
 		form.add(new CheckBox("useTickets"));
-		form.add(new CheckBox("useDocs"));		
+		form.add(new CheckBox("useDocs"));
 		form.add(new CheckBox("showRemoteBranches"));
 		form.add(new CheckBox("showReadme"));
 		form.add(new CheckBox("skipSizeCalculation"));
@@ -367,14 +415,24 @@
 				: StringUtils.flattenStrings(repositoryModel.mailingLists, " "));
 		form.add(new TextField<String>("mailingLists", mailingLists));
 		form.add(indexedBranchesPalette);
+		
+		RadioGroup<AuthorizationControl> group = new RadioGroup<AuthorizationControl>("authorizationControl");
+		Radio<AuthorizationControl> allowAuthenticated = new Radio<AuthorizationControl>("allowAuthenticated", new Model<AuthorizationControl>(AuthorizationControl.AUTHENTICATED));		
+		Radio<AuthorizationControl> allowNamed = new Radio<AuthorizationControl>("allowNamed", new Model<AuthorizationControl>(AuthorizationControl.NAMED));
+		group.add(allowAuthenticated);
+		group.add(allowNamed);
+		form.add(group);
+				
+		form.add(new CheckBox("verifyCommitter"));
+
 		form.add(usersPalette);
 		form.add(teamsPalette);
 		form.add(federationSetsPalette);
 		form.add(preReceivePalette);
-		form.add(new BulletListPanel("inheritedPreReceive", "inherited", GitBlit.self()
+		form.add(new BulletListPanel("inheritedPreReceive", getString("gb.inherited"), GitBlit.self()
 				.getPreReceiveScriptsInherited(repositoryModel)));
 		form.add(postReceivePalette);
-		form.add(new BulletListPanel("inheritedPostReceive", "inherited", GitBlit.self()
+		form.add(new BulletListPanel("inheritedPostReceive", getString("gb.inherited"), GitBlit.self()
 				.getPostReceiveScriptsInherited(repositoryModel)));
 		
 		WebMarkupContainer customFieldsSection = new WebMarkupContainer("customFieldsSection");
@@ -414,31 +472,31 @@
 			if (authenticateAdmin) {
 				if (user == null) {
 					// No Login Available
-					error("Administration requires a login", true);
+					error(getString("gb.errorAdminLoginRequired"), true);
 				}
 				if (isCreate) {
 					// Create Repository
-					if (!user.canAdmin) {
-						// Only Administrators May Create
-						error("Only an administrator may create a repository", true);
+					if (!user.canCreate() && !user.canAdmin()) {
+						// Only administrators or permitted users may create
+						error(getString("gb.errorOnlyAdminMayCreateRepository"), true);
 					}
 				} else {
 					// Edit Repository
-					if (user.canAdmin) {
+					if (user.canAdmin()) {
 						// Admins can edit everything
 						isAdmin = true;
 						return;
 					} else {
 						if (!model.owner.equalsIgnoreCase(user.username)) {
 							// User is not an Admin nor Owner
-							error("Only an administrator or the owner may edit a repository", true);
+							error(getString("gb.errorOnlyAdminOrOwnerMayEditRepository"), true);
 						}
 					}
 				}
 			}
 		} else {
 			// No Administration Permitted
-			error("Administration is disabled", true);
+			error(getString("gb.errorAdministrationDisabled"), true);
 		}
 	}
 

--
Gitblit v1.9.1